@@ -31,6 +31,7 @@
 // Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2013 José Aliste <jaliste@src.gnome.org>
 // Copyright (C) 2014 Ed Porras <ed@moto-research.com>
+// Copyright (C) 2015 Even Rouault <even.rouault@spatialys.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -171,8 +172,8 @@ Catalog::~Catalog() {
       }
     }
     gfree(pages);
-    gfree(pageRefs);
   }
+  gfree(pageRefs);
   names.free();
   dests.free();
   delete destNameTree;
@@ -305,8 +306,14 @@ GBool Catalog::cachePageTree(int page)
     }
 
     pagesSize = getNumPages();
-    pages = (Page **)gmallocn(pagesSize, sizeof(Page *));
-    pageRefs = (Ref *)gmallocn(pagesSize, sizeof(Ref));
+    pages = (Page **)gmallocn_checkoverflow(pagesSize, sizeof(Page *));
+    pageRefs = (Ref *)gmallocn_checkoverflow(pagesSize, sizeof(Ref));
+    if (pages == NULL || pageRefs == NULL ) {
+      error(errSyntaxError, -1, "Cannot allocate page cache");
+      pagesDict->decRef();
+      pagesSize = 0;
+      return gFalse;
+    }
     for (int i = 0; i < pagesSize; ++i) {
       pages[i] = NULL;
       pageRefs[i].num = -1;
@@ -825,24 +832,28 @@ int Catalog::getNumPages()
 	Object pageRootRef;
 	catDict.dictLookupNF("Pages", &pageRootRef);
 
-	error(errSyntaxError, -1, "Pages top-level is a single Page. The document is mal-formet, trying to recover...");
+	error(errSyntaxError, -1, "Pages top-level is a single Page. The document is malformed, trying to recover...");
 
 	Dict *pageDict = pagesDict.getDict();
-	const Ref pageRef = pageRootRef.getRef();
-	Page *p = new Page(doc, 1, pageDict, pageRef, new PageAttrs(NULL, pageDict), form);
-	if (p->isOk()) {
-	  pages = (Page **)gmallocn(1, sizeof(Page *));
-	  pageRefs = (Ref *)gmallocn(1, sizeof(Ref));
+	if (pageRootRef.isRef()) {
+	  const Ref pageRef = pageRootRef.getRef();
+	  Page *p = new Page(doc, 1, pageDict, pageRef, new PageAttrs(NULL, pageDict), form);
+	  if (p->isOk()) {
+	    pages = (Page **)gmallocn(1, sizeof(Page *));
+	    pageRefs = (Ref *)gmallocn(1, sizeof(Ref));
 
-	  pages[0] = p;
-	  pageRefs[0].num = pageRef.num;
-	  pageRefs[0].gen = pageRef.gen;
+	    pages[0] = p;
+	    pageRefs[0].num = pageRef.num;
+	    pageRefs[0].gen = pageRef.gen;
 
-	  numPages = 1;
-	  lastCachedPage = 1;
-	  pagesSize = 1;
+	    numPages = 1;
+	    lastCachedPage = 1;
+	    pagesSize = 1;
+	  } else {
+	    delete p;
+	    numPages = 0;
+	  }
 	} else {
-	  delete p;
 	  numPages = 0;
 	}
       } else {
@@ -852,6 +863,17 @@ int Catalog::getNumPages()
       }
     } else {
       numPages = (int)obj.getNum();
+      if (numPages <= 0) {
+        error(errSyntaxError, -1,
+              "Invalid page count {0:d}", numPages);
+        numPages = 0;
+      } else if (numPages > xref->getNumObjects()) {
+        error(errSyntaxError, -1,
+              "Page count ({0:d}) larger than number of objects ({1:d})",
+              numPages, xref->getNumObjects());
+        numPages = 0;
+      }
+
     }
 
     catDict.free();
